SystemOrganization addCategory: #'Pier-Magma'!
SystemOrganization addCategory: #'Pier-Magma-Tests'!

!PRPierControlPanel methodsFor: '*pier-magma' stamp: 'kph 12/13/2007 01:55'!
doDeleteRepositoryKernel: kernelName

	self doubleConfirm: ('delete kernel from the repository: ',  kernelName)
		thenDo: [ 
				self appSession commit: [ self session pierPersistencies removeKey: kernelName ]
		].! !

!PRPierControlPanel methodsFor: '*pier-magma' stamp: 'kph 1/29/2008 15:44'!
renderPRMagmaPersistencyOn: html

| |

"is database configured?"
 
	(self application configuration hasGroupNamed: #magma)
	  ifFalse: [ 
		html 	text: 'configuration ancestry does not yet include WAMagmaConfiguration '.
		html	anchor callback: [ self call: (self application configurationComponent) ];
						text: '(config)'. 
		^false
	].
 
	self repository at: self kernelChoice name ifPresent: [ :persistedPersistency |
 
		html 
			text: 'Use kernel present in repository '; 
			small: [ html anchor callback: [self doDeleteRepositoryKernel: self kernelChoice name ]; text: '(delete it)' ].		
  	] .

	self repository at: self kernelChoice name ifAbsent: [
		html text: 'Migrate saving this kernel to the repository'
	].

	html break; break.
	
	html table: [ 
		html tableHeading: [ html tableRow: [ 
				html 
					strong: 'Persisted Kernels '; 
					small: [ html anchor callback: [ self repository explore ]; with: '(explore)' ] ] ] .
		self repository associationsDo: [ :each |
			html tableRow: [ html tableData: each ] 
		]	
	]
! !

!PRPierControlPanel methodsFor: '*pier-magma' stamp: 'kph 12/13/2007 04:25'!
repository

	^ self appSession magma rootAs: #PRMagmaRepository! !

!PRStructure methodsFor: '*pier-magma' stamp: 'kph 9/25/2006 04:20'!
maPreserialize

"ensure that there are no empty PRChildren persisted since
reading empties back in generates needless database realisations."

| childrenDecoration | 

childrenDecoration := self decorationOfClass: self childrenDecorationClass ifAbsent: [ ^self ].

childrenDecoration isEmpty ifTrue: [
	self removeDecoration: childrenDecoration.
].
! !

!PRStructure methodsFor: '*pier-magma' stamp: 'kph 9/25/2006 04:15'!
maWantsPreSerialization 
	^ true! !

!PRObject methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:47'!
magmaCopy

^ self shallowCopy postMagmaCopy! !

!PRObject methodsFor: '*pier-magma' stamp: 'kph 9/29/2006 01:00'!
withPersistenceDo: aBlock

| context |

(context := PRCurrentContext value) ifNil: [ ^nil ].

^ aBlock value: context kernel persistency.! !

PRPersistencyTest subclass: #PRMagmaPersistencyTest
	instanceVariableNames: 'location session repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma-Tests'!

!PRMagmaPersistencyTest commentStamp: 'kph 10/6/2006 04:42' prior: 0!
Testing PRMagmaPersistence with a real magma database via MagmaAsTestResource.

Fixture is the kernel, persisted in the repository.
!

!PRMagmaPersistencyTest class methodsFor: 'testing' stamp: 'kph 10/4/2006 11:34'!
isAbstract
	^ false! !

!PRMagmaPersistencyTest class methodsFor: 'testing' stamp: 'kph 10/6/2006 06:39'!
resources

^Array with: TestResourceMagma 
"^Array with: TestResourceMockMagma" ! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/5/2006 21:21'!
actualClass
	^ PRMagmaPersistency! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/6/2006 22:24'!
application

^ self! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/7/2006 01:31'!
fullyRealized: anObject
 
^self fullyRealized: anObject visited: Set new.! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/7/2006 02:33'!
fullyRealized: anObject visited: aSet
 

anObject ifNil: [ ^true ]. 

(aSet includes: anObject) ifTrue: [ ^true ].

(anObject maIsMutatingProxy) ifTrue: [ ^false ].

aSet add: anObject.

1 to: anObject class instSize do: [ :n | (self fullyRealized: (anObject instVarAt: n) visited: aSet) ifFalse: [ ^false ] ].

^true! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/10/2006 09:24'!
initializeRepository

repository := self magma initializeRoot: PRMagmaRepository new.! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/6/2006 06:20'!
magma

^self class resources first current! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 06:37'!
persistencyInstance

	"the default persistency scheme that kernels are created with. When testing magma we always start with a null persistency and export it to the database in setUp."
	
	^ PRNullPersistency new! !

!PRMagmaPersistencyTest methodsFor: 'application-stub' stamp: 'kph 10/6/2006 22:22'!
preferenceAt: key

key == #location ifTrue: [ ^ self magma location ].! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/6/2006 04:29'!
runCase

WACurrentSession use: (self session) during: [ super runCase ].! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/6/2006 04:30'!
runCaseAsFailure: sema

WACurrentSession use: (self session) during: [ super runCaseAsFailure: sema ].! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 06:20'!
session 

^session ifNil: [ 
	session := (WAMagmaSession new setParent: (self application)) setMagmaSession: (self magma session); yourself ]! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 06:28'!
setUp

	super setUp.
 
	self initializeRepository.

	self kernel persistency: (kernel persistency exportTo: PRMagmaPersistency new).
		
	self session stubOut: repository.
	repository := nil.! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/6/2006 04:48'!
tearDown

super tearDown.
! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 01:27'!
testFromPRNullPersistency

"setUp begins with a kernel having PRNullPersistency exports it to PRMagmaPersistency"

| p pierPersistencies |

pierPersistencies := self session pierPersistencies.

p := pierPersistencies at: '####test'.

self assert: (p class == PRMagmaPersistency).
self assert: (p kernel name = '####test').

self assert: (kernel hasRoot not).
self assert: p kernel root notNil.
self assert: (kernel realize == p kernel).! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 10/11/2006 04:05'!
testPathLookupCache

| needle |

needle := PRPathLookup
			start: kernel realize root 
			path: '/information/syntax'.
self assert: ( needle title = 'Syntax' ).

self assert: false.
! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 01:28'!
testRenameKernel

	super testRenameKernel.

	self assert: (kernel realize == (self session pierPersistencies at: '####new') kernel).

	self assert: kernel realize name = '####new'.

	self assert: (self session pierPersistencies at: '####test' ifAbsent: [ true ]).! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 01:28'!
testRenameKernelFailDueToAlreadyPersisted
	
	 

	self session pierPersistencies at: '####new' put: (PRKernel new name: '####new'; yourself).

	self should: [ kernel persistency renameKernelTo: '####new' ] raise: Error.

 ! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 01:28'!
testRenameKernelFailDueToNameTaken

	super testRenameKernelFailDueToNameTaken.

	self shouldnt: [ self session pierPersistencies at: '####test' ] raise: Error.
	self should: [ self session pierPersistencies at: '####new' ] raise: Error.

 ! !

!PRMagmaPersistencyTest methodsFor: 'private' stamp: 'kph 4/21/2007 01:28'!
testToPRNullPersistency

 | p pierPersistencies |

 
kernel persistency: (kernel persistency exportTo: PRNullPersistency new).
 

pierPersistencies := self session pierPersistencies.
p := pierPersistencies at: '####test'.
self assert: (p class == PRMagmaPersistency).
self assert: (p kernel name = '####test').

self assert: (kernel hasRoot).
self assert: (kernel persistency class == PRNullPersistency).
self assert: (kernel realize == kernel).

self assert: (self fullyRealized: kernel). ! !

PRImagePersistency subclass: #PRBackupToMagmaPersistency
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRBackupToMagmaPersistency methodsFor: 'as yet unclassified' stamp: 'kph 10/11/2006 02:25'!
initialize
	super initialize.
	duration := 2 minutes! !

!PRBackupToMagmaPersistency methodsFor: 'as yet unclassified' stamp: 'kph 12/13/2007 05:59'!
session

	^ WACurrentSession value.
 ! !

!PRBackupToMagmaPersistency methodsFor: 'as yet unclassified' stamp: 'kph 10/11/2006 02:26'!
setKernel: aKernel
	super setKernel: aKernel.
	aKernel ifNotNil: [ self snapshot ].! !

!PRBackupToMagmaPersistency methodsFor: 'as yet unclassified' stamp: 'kph 1/28/2008 21:42'!
snapshot

	| db |
	self session magma in: [ :magma |
		
		db := magma rootAs: PRMagmaRepository.
		
		magma inTransaction ifFalse: [ magma begin ].

		db at: self kernel name put: self.
		
		magma commitAndBegin
	].! !

!MAProxyObject methodsFor: '*pier-magma' stamp: 'kph 1/30/2008 12:03'!
instVarAt: index 
 
	<primitive: 73>
	"Access beyond fixed variables."
	^self basicAt: index - self class instSize		! !

!MAProxyObject methodsFor: '*pier-magma' stamp: 'kph 1/30/2008 12:03'!
instVarAt: anInteger put: anObject 
 
	<primitive: 74>
	"Access beyond fixed fields"
	^self basicAt: anInteger - self class instSize put: anObject! !

!MAProxyObject methodsFor: '*pier-magma-override' stamp: 'kph 1/30/2008 12:14'!
isNil
	"This method is required to properly return ==true== if the ==realSubject== is ==nil==."

 
	realSubject isInteger ifTrue: [ ^ false ].
	
	^ self realSubject isNil! !

MagmaDictionaryRoot subclass: #PRMagmaRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaRepository commentStamp: '<historical>' prior: 0!
I am used as the root object in a magma repository. 
I am subclassed from PRObject so that I may have properties.

----
To Create Repository:

There is a GUI provided in the Magma-seaside package which is available within seaside/config to create a repository.

To manually create a repository directory called 'magma-pier' in the default directory:

MagmaRepositoryController create: 'magma-pier' root: PRMagmaRepository new.

Again there is a GUI available in the 'MagmaControlPanel' for persisting PRKernels.
Manually we tell the default repository to persist an existing pier kernel referenced by its name like so.  

| session |
session := MagmaSession openLocal: 'magma-pier'.
session connectAs: 'local-squeak'.
session commit: [ 
PRMagmaRepository persist: (PRKernel instanceNamed: 'home').
].
session disconnectAndClose.

The existing persistence structure is migrated into the magma repository. The framework for acheiving this is provided in the protocol 'translating'. Although at present this does nothing, it is here in order to allow PRMagmaPersistency to tweak the model that it it persisting. For example we may choose to use indexed MagmaCollections to store the history. The Pier framework need not be aware of any Magma specifics, hence the double dispatch of exportToMagma methods which refer back to Magma specific importFrom methods on PRMagmaPersistency.

 

!

!PRMagmaRepository class methodsFor: 'read strategy' stamp: 'kph 12/12/2007 23:59'!
readPathSkeletonInOneHit

"the path skeleton is all of the data needed to traverse the heirarchical path of the pier kernel
 - if this is set to false then each level down in the path traversal will be a separate object realisation
 - this may be desireable for small to medium sizred wiki's"

^false! !

!PRMagmaRepository class methodsFor: 'read strategy' stamp: 'kph 1/30/2008 01:32'!
readStrategyBuildOn: strategy

"set the read strategy for this application"

strategy minimumDepth: 0;
	"	forVariableNamed: 'store' onAny: PRMagmaRepository readToDepth: 2 ;"
		forVariableNamed: 'kernel' onAny: PRPersistency readToDepth: 1;
		onAny: PRKernel readToDepth: 1 ;
		forVariableNamed: 'decorations' onAny: PRDecorated readToDepth: 1; 
		forVariableNamed: 'title' onAny: PRStructure readToDepth: 1; 

		forVariableNamed: 'children' onAny: PRChildren readToDepth: 
			(self readPathSkeletonInOneHit ifTrue: [ 1 ] ifFalse: [ 0 ]);

		onAny: PRDocumentItem readToDepth: 1; "reads all page items in one hit"

		onAny: PRComponent readToDepth: 1;
yourself.! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 12/13/2007 00:02'!
persist: kernel  

	"we make a new kernel instance to go into the repository,
	we assign its persistency manager, and root structure.

	This process includes an opportunity to munge its model if needed
	or to import from another persistency scheme.

	Then we turn the image PRKernel into a stub which is used to obtain the persisted kernel via #realize.
	"

	| persistedKernel | 

	persistedKernel := PRKernel new name: kernel name; yourself.

	persistedKernel persistency: ( kernel persistency exportToMagma: PRMagmaPersistency new ).
	persistedKernel root: kernel root.

	self at: kernel name put:  kernel persistency.

	"stub out image kernel"
	kernel persistency: PRMagmaPersistency new.
	kernel root: nil.! !

PRMagmaRepository subclass: #PRMagmaRepositoryReadWholeKernel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaRepositoryReadWholeKernel class methodsFor: 'as yet unclassified' stamp: 'kph 12/13/2007 05:33'!
modelClass

	^ PRMagmaRepository! !

!PRMagmaRepositoryReadWholeKernel class methodsFor: 'as yet unclassified' stamp: 'kph 1/30/2008 01:32'!
readStrategyBuildOn: rs

	rs onAny: PRKernel readToDepth: 99999.
	
	 ^ rs! !

!PRMagmaRepositoryReadWholeKernel methodsFor: 'as yet unclassified' stamp: 'kph 12/13/2007 05:33'!
isCached

	^ false! !

Object subclass: #PRCachedPath
	instanceVariableNames: 'fullPath structure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRCachedPath class methodsFor: 'as yet unclassified' stamp: 'kph 10/10/2006 10:58'!
path: aString structure: aStructure

^self new 
	setFullPath: aString structure: aStructure; 
	yourself.! !

!PRCachedPath methodsFor: 'as yet unclassified' stamp: 'kph 10/10/2006 10:58'!
fullPath

^fullPath! !

!PRCachedPath methodsFor: 'as yet unclassified' stamp: 'kph 10/10/2006 10:58'!
setFullPath: aString structure: aStructure

fullPath := aString.
structure := aStructure.! !

!PRCachedPath methodsFor: 'as yet unclassified' stamp: 'kph 10/10/2006 10:57'!
structure

^structure! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:26'!
maPreserialize

"ensure that properties are always zapped before persisting"
	properties := nil.! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:25'!
maWantsPreSerialization 
	^ true! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:27'!
postMagmaCopy

	properties := nil.	 
	self setCommand: self command magmaCopy.! !

!PRCommand methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:28'!
postMagmaCopy

	answer ifNotNil: [ answer := answer magmaCopy ].! !

PRPersistency subclass: #PRMagmaPersistency
	instanceVariableNames: 'snapshot contexts paths'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaPersistency commentStamp: 'kph 9/14/2006 03:33' prior: 0!
I am stored in the PRMagmaRepository as the base object for each PRKernel.
I store the current #kernel
I store a historical #snapshot
I store a log of changes, a collection of #contexts, whose commands can be replayed to obtain a complete history.

----
In Pier, PRContext objects may cache #properties in relation to the current seaside display context session and components. Persisting these runtime non-model objects is not a good idea since they may include references to the current magma session which will break the persistency. To avoid this we send #magmaCopy to all instances of PRContext, before saving them into the log of #contexts. This also applies to the 'context's command's answer context'.

If one of these persisted contexts is used and by chance has its properties repopulated, then PRContext-#maPreSerialize zaps #properties as well before the object would be serialised and persisted again. 



!

!PRMagmaPersistency class methodsFor: 'as yet unclassified' stamp: 'kph 10/4/2006 11:36'!
isAbstract

^false! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 4/21/2007 01:22'!
discarded
 
 
self session commit: [ self session pierPersistencies removeKey: self kernel name ].! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:18'!
do: aBlock 
	 contexts do: [ :each | aBlock value: (each setKernel: self kernel) ]! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/12/2007 23:22'!
execute: aCommand
	"Execute aCommand using the persistency of the receiver. Subclasses might override this method, e.g. to create a transitional logic or to commit changes after execution. Make sure that you always perform the method <code>#apply:</code> when overriding this code."

	"magma does not need this method at all when using an 'Automatic' Helper"

	self session magma isAuto 
		ifFalse: [
			self mutex critical: [ self session commit: [ self apply: aCommand ] ]
		]
		ifTrue: [ 
			super execute: aCommand
		].
	 ! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 10/6/2006 09:48'!
exportTo: aPRPersistency

"at some point we need to realise the whole kernel into memory, at least, 
 possibly history too if the new class wants it if we do it"

^aPRPersistency fromPRMagmaPersistency: self ! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 12/13/2007 05:40'!
exportToPRPersistency: aPRPersistency

"at some point we need to realise the whole kernel into memory, at least, 
 possibly history too if the new class wants it if we do it"

	| persisted newKernel |

	newKernel := (PRKernel new) 
					name: self kernel name;
					persistency: aPRPersistency;
					yourself.

	"obtain a copy of the real kernel from magma"
	persisted := ((WAMagma forApp: self session application name) rootAs: PRMagmaRepositoryReadWholeKernel) at: self kernel name.
	
	newKernel root: persisted kernel root.		
	
	^ aPRPersistency
! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 12/13/2007 05:06'!
fromPRPersistency: aPersistency

	| myName | 

	myName := aPersistency kernel name.

	self repository at: myName ifAbsent: [ 
		PRKernel new 
			name: myName; 
		   	persistency: self;
			root: aPersistency kernel root;
			yourself.

		self session magma commit: [ self repository at: myName put: self].
	].
 
	^ PRMagmaPersistency new setKernel: (PRKernel new name: myName)
	
	 ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 4/20/2007 03:40'!
initialize
	super initialize.
	contexts := OrderedCollection new.! !

!PRMagmaPersistency methodsFor: 'cache' stamp: 'kph 9/29/2006 00:15'!
internalLink: aLink

"we are informed of an internal link so we may cache it"
 ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/14/2006 02:11'!
log: aContext

	 contexts add: aContext magmaCopy ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 10/11/2006 03:46'!
paths

^paths ifNil: [ 
	paths := MagmaCollection new addIndex:
			((MaSearchStringIndex attribute: #fullPath) keySize: 128; yourself);
			yourself.
].! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/12/2007 23:15'!
realizeKernel

	"I return the kernel instance from the repository under my kernel's name (my own kernel is likely to be a stub)"
 
	self repository at: (self kernel name) ifPresent: [ :p | ^ p kernel ].

	self error: ('not found in database: ', self kernel name) ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/12/2007 23:49'!
renameKernelTo: newName

	| oldName  old |

	self validateNameAvailable: newName.

	oldName := self kernel name.
	
	self session magma commit: [
		self repository in: [ :repo |
			old := repo at: oldName. 
			old kernel name: newName.
			repo at: newName put: old.
			repo removeKey: oldName.	
			self kernel name: newName.
		].
	].

! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/12/2007 23:11'!
repository

	^ self session magma rootAs: PRMagmaRepository ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:19'!
reverseDo: aBlock 
	contexts reverseDo: [ :each | aBlock value: (each setKernel: self kernel) ]! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/12/2007 23:26'!
session

	^  WACurrentSession value
 ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 10/6/2006 23:59'!
setKernel: aKernel

	"when we are removed from persistency nil out the kernel's root (it should be nil anyway)
	so that the new persistency scheme will supply the kernel with data"

	aKernel ifNil: [ 
		kernel root: nil.
	] ifNotNil: [	
		aKernel root: nil. "nil out the kernel's data since we have it persisted"
	].
	
	kernel := aKernel.! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/29/2006 06:23'!
snapshot
	snapshot := self kernel veryDeepCopy.
	contexts := OrderedCollection new! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 12/13/2007 03:14'!
validateNameAvailable: newName
 
	super validateNameAvailable: newName.
	self repository at: newName 
		ifPresent: [ :p | self error: 'A kernel with that name already exists in Magma repository' ]. 
! !

!PRPersistency methodsFor: '*pier-magma' stamp: 'kph 4/21/2007 01:59'!
maPreserialize

"ensure that mutex is zapped before persisting, then at least it will be in a predictable state when relaised."
	"mutex := nil."! !

!PRPersistency methodsFor: '*pier-magma' stamp: 'kph 4/21/2007 01:58'!
maWantsPreSerialization 
	^ true! !
